/*
 * Copyright (c) 2019 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <glib.h>
#include <aul.h>
#include <aul_comp_status.h>

#include "rua_private.h"
#include "rua_info.h"
#include "rua_context.h"
#include "rua_manager.h"

struct rua_context_s {
	char *app_id;
	char *instance_id;
	char *component_id;
};

static void __destroy_rua_context(struct rua_context_s *context)
{
	free(context->component_id);
	free(context->instance_id);
	free(context->app_id);
	free(context);
}

static struct rua_context_s *__create_rua_context(char *app_id,
		char *instance_id, char *component_id)
{
	struct rua_context_s *context;

	context = calloc(1, sizeof(struct rua_context_s));
	if (!context) {
		_E("Out of memory");
		return NULL;
	}

	context->app_id = app_id;
	context->instance_id = instance_id;
	context->component_id = component_id;

	return context;
}

int rua_context_create_from_rua_info(rua_info_h info,
		rua_context_h *context)
{
	struct rua_context_s *handle;
	char *app_id;
	char *instance_id;
	char *component_id;
	int ret;

	if (!info || !context) {
		_E("Invalid parameter");
		return RUA_ERROR_INVALID_PARAMETER;
	}

	ret = rua_info_get_app_id(info, &app_id);
	if (ret != RUA_ERROR_NONE)
		return ret;

	ret = rua_info_get_instance_id(info, &instance_id);
	if (ret != RUA_ERROR_NONE) {
		free(app_id);
		return ret;
	}

	ret = rua_info_get_component_id(info, &component_id);
	if (ret != RUA_ERROR_NONE) {
		free(instance_id);
		free(app_id);
		return ret;
	}

	handle = __create_rua_context(app_id, instance_id, component_id);
	if (!handle) {
		free(component_id);
		free(instance_id);
		free(app_id);
		return RUA_ERROR_OUT_OF_MEMORY;
	}

	*context = handle;

	return RUA_ERROR_NONE;
}

API int rua_context_destroy(rua_context_h context)
{
	if (!context) {
		_E("Invalid parameter");
		return RUA_ERROR_INVALID_PARAMETER;
	}

	__destroy_rua_context(context);

	return RUA_ERROR_NONE;
}

API int rua_context_get_app_id(rua_context_h context, char **app_id)
{
	if (!context || !app_id) {
		_E("Invalid parameter");
		return RUA_ERROR_INVALID_PARAMETER;
	}

	*app_id = strdup(context->app_id);
	if (*app_id == NULL) {
		_E("Out of memory");
		return RUA_ERROR_OUT_OF_MEMORY;
	}

	return RUA_ERROR_NONE;
}

API int rua_context_get_instance_id(rua_context_h context, char **instance_id)
{
	if (!context || !instance_id) {
		_E("Invalid parameter");
		return RUA_ERROR_INVALID_PARAMETER;
	}

	if (context->instance_id) {
		*instance_id = strdup(context->instance_id);
		if (*instance_id == NULL) {
			_E("Out of memory");
			return RUA_ERROR_OUT_OF_MEMORY;
		}
	} else {
		*instance_id = NULL;
	}

	return RUA_ERROR_NONE;
}

API int rua_context_get_component_id(rua_context_h context, char **component_id)
{
	if (!context || !component_id) {
		_E("Invalid parameter");
		return RUA_ERROR_INVALID_PARAMETER;
	}

	if (context->component_id) {
		*component_id = context->component_id;
		if (*component_id == NULL) {
			_E("Out of memory");
			return RUA_ERROR_OUT_OF_MEMORY;
		}
	} else {
		*component_id = NULL;
	}

	return RUA_ERROR_NONE;
}

static int __convert_aul_error(int error)
{
	switch (error) {
	case AUL_R_EINVAL:
		return RUA_ERROR_INVALID_PARAMETER;
	case AUL_R_EILLACC:
		return RUA_ERROR_PERMISSION_DENIED;
	case AUL_R_ENOAPP:
		return RUA_ERROR_NO_SUCH_APP;
	default:
		return RUA_ERROR_IO_ERROR;
	}
}

int rua_context_is_running(rua_context_h context, bool *running)
{
	int ret;

	if (!context || !running) {
		_E("Invalid parameter");
		return RUA_ERROR_INVALID_PARAMETER;
	}

	if (context->component_id) {
		ret = aul_comp_is_running(context->instance_id, running);
	} else if (context->instance_id) {
		ret = aul_app_is_running_with_instance_id(context->app_id,
				context->instance_id, running);
	} else {
		ret = aul_app_is_running(context->app_id);
		if (ret >= 0)
			*running = (bool)ret;
	}

	if (ret < 0) {
		_E("Failed to check running state");
		return __convert_aul_error(ret);
	}

	return RUA_ERROR_NONE;
}

int rua_context_resume(rua_context_h context)
{
	int ret;

	if (!context) {
		_E("Invalid parameter");
		return RUA_ERROR_INVALID_PARAMETER;
	}

	if (context->component_id) {
		ret = aul_comp_resume(context->instance_id);
	} else if (context->instance_id) {
		ret = aul_resume_app_by_instance_id(context->app_id,
				context->instance_id);
	} else {
		ret = aul_resume_app(context->app_id);
	}

	if (ret < 0) {
		_E("Failed to send the resume request. error(%d)", ret);
		return __convert_aul_error(ret);
	}

	return RUA_ERROR_NONE;
}

int rua_context_terminate(rua_context_h context)
{
	int ret;

	if (!context) {
		_E("Invalid parameter");
		return RUA_ERROR_INVALID_PARAMETER;
	}

	if (context->component_id) {
		ret = aul_comp_terminate(context->instance_id);
	} else if (context->instance_id) {
		ret = aul_terminate_app_with_instance_id(context->app_id,
				context->instance_id);
	} else {
		ret = aul_terminate_app(context->app_id);
	}

	if (ret < 0) {
		_E("Failed to send the terminate request. error(%d)", ret);
		return __convert_aul_error(ret);
	}

	return RUA_ERROR_NONE;
}
