/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include <unistd.h>
#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <dlog.h>
#include <sqlite3.h>

#include "db-schema.h"
#include "rua_util.h"
#include "rua_private.h"

#define WIN_SCORE 100
#define LOSE_SCORE_RATE 0.7f

int __rua_stat_insert(sqlite3 *db, char *caller, char *rua_stat_tag)
{
	static const char query[] =
		"INSERT INTO rua_panel_stat ("
		"  caller_panel, rua_stat_tag, score) "
		"VALUES (?, ?, ?)";
	int r;
	sqlite3_stmt *stmt;
	int idx = 1;

	r = sqlite3_prepare(db, query, sizeof(query), &stmt, NULL);
	if (r != SQLITE_OK) {
		LOGE("prepare failed: %s", sqlite3_errmsg(db));
		return -1;
	}

	__BIND_TEXT(db, stmt, idx++, caller);
	__BIND_TEXT(db, stmt, idx++, rua_stat_tag);
	__BIND_INT(db, stmt, idx++, WIN_SCORE);

	r = sqlite3_step(stmt);
	if (r != SQLITE_DONE) {
		LOGE("step failed: %s", sqlite3_errmsg(db));
		sqlite3_finalize(stmt);
		return -1;
	}

	sqlite3_finalize(stmt);

	return r;
}

int __rua_stat_lose_score_update(sqlite3 *db, char *caller, char *rua_stat_tag)
{
	static const char query[] =
		"UPDATE rua_panel_stat SET score=score*? "
		"WHERE caller_panel=? AND rua_stat_tag!=?";
	int r;
	sqlite3_stmt *stmt;
	int idx = 1;

	LOGD("lose score update sql: %s, %s", caller, rua_stat_tag);
	r = sqlite3_prepare(db, query, sizeof(query), &stmt, NULL);
	if (r != SQLITE_OK) {
		LOGE("prepare failed: %s", sqlite3_errmsg(db));
		return r;
	}

	__BIND_DOUBLE(db, stmt, idx++, LOSE_SCORE_RATE);
	__BIND_TEXT(db, stmt, idx++, caller);
	__BIND_TEXT(db, stmt, idx++, rua_stat_tag);

	r = sqlite3_step(stmt);
	if (r != SQLITE_DONE) {
		LOGE("step failed: %s", sqlite3_errmsg(db));
		sqlite3_finalize(stmt);
		return r;
	}

	sqlite3_finalize(stmt);

	return r;
}

int __rua_stat_win_score_update(sqlite3 *db, char *caller, char *rua_stat_tag)
{
	static const char query[] =
		"UPDATE rua_panel_stat SET score=score+? "
		"WHERE caller_panel=? AND rua_stat_tag=?";
	int r;
	sqlite3_stmt *stmt;
	int idx = 1;

	LOGD("win score update sql: %s, %s", caller, rua_stat_tag);

	r = sqlite3_prepare(db, query, sizeof(query), &stmt, NULL);
	if (r != SQLITE_OK) {
		LOGE("prepare failed: %s", sqlite3_errmsg(db));
		return r;
	}

	__BIND_INT(db, stmt, idx++, WIN_SCORE);
	__BIND_TEXT(db, stmt, idx++, caller);
	__BIND_TEXT(db, stmt, idx++, rua_stat_tag);

	r = sqlite3_step(stmt);
	if (r != SQLITE_DONE) {
		LOGE("step failed: %s", sqlite3_errmsg(db));
		sqlite3_finalize(stmt);
		return r;
	}

	sqlite3_finalize(stmt);

	return r;
}

static int __create_table(sqlite3 *db)
{
	int r;
	char *errmsg = NULL;

	r = sqlite3_exec(db, CREATE_RUA_STAT_TABLE, NULL, NULL, &errmsg);
	if (r != SQLITE_OK) {
		LOGE("create table failed: %s", errmsg);
		sqlite3_free(errmsg);
		return -1;
	}

	return 0;
}

API int rua_stat_usr_db_update(char *caller, char *rua_stat_tag, uid_t uid)
{
	int r;
	int affected_rows = 0;
	sqlite3 *db;

	LOGD("rua_stat_update start");

	if (caller == NULL || rua_stat_tag == NULL) {
		LOGE("invalid parameter");
		return -1;
	}

	r = _rua_util_open_db(&db, SQLITE_OPEN_CREATE | SQLITE_OPEN_READWRITE,
			uid, RUA_STAT_DB_NAME);
	if (r == -1) {
		LOGE("open rua stat db failed");
		return -1;
	}

	r = __create_table(db);
	if (r == -1) {
		LOGE("create rua stat tabale failed");
		return -1;
	}

	r = __rua_stat_lose_score_update(db, caller, rua_stat_tag);
	if (r != SQLITE_DONE) {
		LOGE("__rua_stat_lose_score_insert failed");
		return -1;
	}

	r = __rua_stat_win_score_update(db, caller, rua_stat_tag);
	affected_rows = sqlite3_changes(db);
	if ((r != SQLITE_DONE) || (affected_rows == 0)) {
		r = __rua_stat_insert(db, caller, rua_stat_tag);

		if (r != SQLITE_DONE) {
			LOGE("__rua_stat_insert failed");
			return -1;
		}
	}

	sqlite3_close_v2(db);
	LOGD("rua_stat_update done");

	return r;
}

API int rua_stat_db_update(char *caller, char *rua_stat_tag)
{
	return rua_stat_usr_db_update(caller, rua_stat_tag, getuid());
}
